
import java.io.* ;
import java.net.* ;
import java.rmi.* ;
import java.rmi.registry.* ;
import java.rmi.server.UnicastRemoteObject ;

import java.util.* ;

// Implements synchronous (occam-channel-like) communication of remote
// object references.

// In this implementation, only the "putting end" can be remote from the
// channel object.

class RemoteChannelImpl extends UnicastRemoteObject implements RemoteChannel {

    RemoteChannelImpl() throws RemoteException {
        waiting = false;
    }

    public synchronized void put(Remote obj) throws RemoteException {
        this.obj = obj;
        if (waiting) {
            waiting = false;
            notify();
        }
        else {
            waiting = true;
            try {
                wait(0);
            } catch (InterruptedException e) {
                System.err.println("Unexpected exception: " + e.getMessage());
                e.printStackTrace();
                System.exit(1);
            }
        }
    }

    public synchronized Remote get() {
        if (waiting) {
            waiting = false;
            notify();
        }
        else {
            waiting = true;
            try {
                wait(0);
            } catch (InterruptedException e) {
                System.err.println("Unexpected exception: " + e.getMessage());
                e.printStackTrace();
                System.exit(1);
            }
        }

        return obj ;
    }

    boolean waiting;
    Remote obj;
}
